'''
Stores and reads a model object.

Functions:
- save_data:    save a model object.
- read_data:    read a model object.
'''


from . import simulation

import json
import gzip
import os


def save_data(mod, dirs = '', print_msg = True):
    '''
    Saves a model object. Data will be stored at dirs/data.json.gz

    Inputs:
    - mod:        Your model object.
    - dirs:       Where to save it.
    - print_msg:  Whether to print message after saving.
    '''

    try:
        _ = mod.N
    except AttributeError:
        raise ValueError('mod is not a model object')

    if dirs != '':
        # add slash '/'
        if dirs[:-1] != '/':
            dirs += '/'
        if not os.path.exists(dirs):
            os.makedirs(dirs)
        
    data = []
    
    inputs1 = []
    inputs1.append(mod.N)
    inputs1.append(mod.M)
    inputs1.append(mod.maxtime)
    inputs1.append(mod.record_itv)
    inputs1.append(mod.sim_time)
    inputs1.append(mod.boundary)
    inputs1.append(mod.I.tolist())
    inputs1.append(mod.X.tolist())
    inputs1.append(mod.P.tolist())
    data.append(inputs1)

    inputs2 = []
    inputs2.append(mod.print_pct)
    inputs2.append(mod.seed)
    inputs2.append(mod.UV_dtype)
    inputs2.append(mod.pi_dtype)
    data.append(inputs2)

    # skipped rng
    
    outputs = []
    outputs.append(mod.max_record)
    outputs.append(mod.compress_itv)
    outputs.append(mod.U.tolist())
    outputs.append(mod.V.tolist())
    outputs.append(mod.U_pi.tolist())
    outputs.append(mod.V_pi.tolist())
    # H&V_pi_total are not saved, will be calculated when reading the data
    data.append(outputs)
    
    data_json = json.dumps(data)
    data_bytes = data_json.encode('utf-8')
    data_dirs = dirs + 'data.json.gz'
    
    with gzip.open(data_dirs, 'w') as f:
        f.write(data_bytes)
    
    if print_msg:
        print('data saved: ' + data_dirs)



def read_data(dirs):
    '''
    Reads and returns a model object.

    Inputs:
    - dirs:       where to read from, just provide the folder-subfolder names. Don't include 'data.json.gz'
    - print_msg:  this function prints a message when the mod.compress_itv != None. Setting print_msg = False will skip ignore this message.

    Returns:
    - mod: a piegy.model.model object read from the data.
    '''
    
    if dirs != '':
        # add slash '/'
        if dirs[:-1] != '/':
            dirs += '/'
        if not os.path.exists(dirs):
            raise FileNotFoundError('dirs not found: ' + dirs)
            
    if not os.path.isfile(dirs + 'data.json.gz'):
        raise FileNotFoundError('data not found in ' + dirs)
    
    with gzip.open(dirs + 'data.json.gz', 'r') as f:
        data_bytes = f.read()
    data_json = data_bytes.decode('utf-8')
    data = json.loads(data_json)

    # inputs
    try:
        mod = simulation.model(N = data[0][0], M = data[0][1], maxtime = data[0][2], record_itv = data[0][3],
                            sim_time = data[0][4], boundary = data[0][5], I = data[0][6], X = data[0][7], P = data[0][8], 
                            print_pct = data[1][0], seed = data[1][1], UV_dtype = data[1][2], pi_dtype = data[1][3])
    except:
        raise ValueError('Invalid input parameters saved in data')

    # outputs
    try:
        mod.set_data(data_empty = False, max_record = data[2][0], compress_itv = data[2][1], 
                    U = data[2][2], V = data[2][3], U_pi = data[2][4], V_pi = data[2][5])
    except:
        raise ValueError('Invalid model results saved in data')
    
    return mod





